import pytest
import iuliia


@pytest.mark.parametrize(
    "source,expected",
    [
        (
            "Юлия, съешь ещё этих мягких французских булок из Йошкар-Олы, да выпей алтайского чаю",
            "Yuliya, s''esh' eshhyo etix myagkix francuzskix bulok iz Joshkar-Oly, "
            "da vypej altajskogo chayu",
        ),
    ],
)
def test_gost_7034(source, expected):
    assert iuliia.translate(source, iuliia.GOST_7034) == expected



"""
MVD 310-1997 transliteration schema.
https://dangry.ru/iuliia/mvd-310/
"""

from iuliia.mapping import BASE_MAPPING
from iuliia.schema import Schema

EN_MAPPING = {
    **BASE_MAPPING,
    **{
        "ё": "e",
        "ж": "zh",
        "й": "y",
        "х": "kh",
        "ц": "ts",
        "ч": "ch",
        "ш": "sh",
        "щ": "shch",
        "ъ": '"',
        "ы": "y",
        "ь": "'",
        "э": "e",
        "ю": "yu",
        "я": "ya",
    },
}

EN_PREV_MAPPING = {"ье": "ye", "ъе": "ye"}
EN_NEXT_MAPPING = {"ье": "", "ъе": ""}

FR_MAPPING = {
    **BASE_MAPPING,
    **{
        "ё": "e",
        "ж": "j",
        "й": "i",
        "у": "ou",
        "х": "kh",
        "ц": "ts",
        "ч": "tch",
        "ш": "ch",
        "щ": "chtch",
        "ъ": "",
        "ы": "y",
        "ь": "",
        "э": "e",
        "ю": "iou",
        "я": "ia",
    },
}

FR_PREV_MAPPING = {"ге": "ue", "ги": "ui", "гы": "uy", "ье": "ie", "кс": "x"}
FR_NEXT_MAPPING = {"кс": ""}
FR_ENDING_MAPPING = {"ин": "ine"}

# This schema defines the following rule for the French mapping:
# > `С` between two vowels → `SS`
# There is no such rule in other schemas, and MVD-310 itself is deprecated,
# so I decided to ignore this specific rule for the sake of code simplicity.

MVD_310 = Schema(EN_MAPPING, prev_mapping=EN_PREV_MAPPING, next_mapping=EN_NEXT_MAPPING)
MVD_310_FR = Schema(
    FR_MAPPING,
    prev_mapping=FR_PREV_MAPPING,
    next_mapping=FR_NEXT_MAPPING,
    ending_mapping=FR_ENDING_MAPPING,
)



"""
Transliteration schema base features.
"""

from .mapping import LetterMapping, PrevMapping, NextMapping, EndingMapping


class Schema:
    """
    Transliteration schema. Defines the way to translate individual letters.
    """

    def __init__(
        self,
        mapping: dict,
        prev_mapping: dict = None,
        next_mapping: dict = None,
        ending_mapping: dict = None,
    ):
        self.map = LetterMapping(mapping)
        self.prev_map = PrevMapping(prev_mapping or {})
        self.next_map = NextMapping(next_mapping or {})
        self.ending_map = EndingMapping(ending_mapping or {})

    def translate_letter(self, prev, curr, next_):
        """Translate ``curr`` letter according to schema mappings.

        ``prev`` (the previous letter) and ``next_`` (the next one)
        are taken into consideration according to corresponding mappings.

        """
        letter = self.prev_map.get(prev + curr)
        if letter is None:
            letter = self.next_map.get(curr + next_)
        if letter is None:
            letter = self.map.get(curr, curr)
        return letter

    def translate_ending(self, ending):
        """Translate word ending according to schema mapping."""
        return self.ending_map.get(ending)



"""
ISO/R 9:1968 transliteration schema.
https://dangry.ru/iuliia/iso-9-1968/
"""

from iuliia.mapping import BASE_MAPPING
from iuliia.schema import Schema

MAPPING = {
    **BASE_MAPPING,
    **{
        "ё": "ë",
        "ж": "ž",
        "й": "j",
        "х": "h",
        "ц": "c",
        "ч": "č",
        "ш": "š",
        "щ": "šč",
        "ъ": "ʺ",
        "ы": "y",
        "ь": "ʹ",
        "э": "ė",
        "ю": "ju",
        "я": "ja",
    },
}

ALT_MAPPING = {
    **BASE_MAPPING,
    **{
        "ё": "ë",
        "ж": "zh",
        "и": "y",
        "й": "ĭ",
        "х": "kh",
        "ц": "ts",
        "ч": "ch",
        "ш": "sh",
        "щ": "shch",
        "ъ": "ʺ",
        "ы": "y",
        "ь": "ʹ",
        "э": "ė",
        "ю": "yu",
        "я": "ya",
    },
}

ISO_9_1968 = Schema(MAPPING)
ISO_9_1968_ALT = Schema(ALT_MAPPING)



import pytest
import iuliia


@pytest.mark.parametrize(
    "source,expected",
    [
        (
            "Юлия, съешь ещё этих мягких французских булок из Йошкар-Олы, да выпей алтайского чаю",
            "Ûliâ, sʺešʹ eŝё ètih mâgkih francuzskih bulok iz Joškar-Oly, da vypej altajskogo čaû",
        ),
    ],
)
def test_gost_16876(source, expected):
    assert iuliia.translate(source, iuliia.GOST_16876) == expected


@pytest.mark.parametrize(
    "source,expected",
    [
        (
            "Юлия, съешь ещё этих мягких французских булок из Йошкар-Олы, да выпей алтайского чаю",
            "Julija, s\"esh' eshhjo ehtikh mjagkikh francuzskikh bulok iz Jjoshkar-Oly, "
            "da vypejj altajjskogo chaju",
        ),
    ],
)
def test_gost_16876_alt(source, expected):
    assert iuliia.translate(source, iuliia.GOST_16876_ALT) == expected



import pytest
import iuliia


@pytest.mark.parametrize(
    "source,expected",
    [
        (
            "Юлия, съешь ещё этих мягких французских булок из Йошкар-Олы, да выпей алтайского чаю",
            "I͡ulii͡a, sʺeshʹ eshchё ėtikh mi͡agkikh frant͡suzskikh bulok iz Ĭoshkar-Oly, "
            "da vypeĭ altaĭskogo chai͡u",
        ),
    ],
)
def test_ala_lc(source, expected):
    assert iuliia.translate(source, iuliia.ALA_LC) == expected


@pytest.mark.parametrize(
    "source,expected",
    [
        (
            "Юлия, съешь ещё этих мягких французских булок из Йошкар-Олы, да выпей алтайского чаю",
            "Iuliia, s\"esh' eshche etikh miagkikh frantsuzskikh bulok iz Ioshkar-Oly, "
            "da vypei altaiskogo chaiu",
        ),
    ],
)
def test_ala_lc_alt(source, expected):
    assert iuliia.translate(source, iuliia.ALA_LC_ALT) == expected



"""
Transliterate Cyrillic → Latin in every possible way
"""

# flake8: noqa
from .engine import translate
from .schema import Schema
from .schemas import (
    Schemas,
    ALA_LC,
    ALA_LC_ALT,
    BGN_PCGN,
    BGN_PCGN_ALT,
    BS_2979,
    BS_2979_ALT,
    GOST_16876,
    GOST_16876_ALT,
    GOST_52290,
    GOST_52535,
    GOST_7034,
    GOST_779,
    GOST_779_ALT,
    ICAO_DOC_9303,
    ISO_9_1954,
    ISO_9_1968,
    ISO_9_1968_ALT,
    MVD_310,
    MVD_310_FR,
    MVD_782,
    SCIENTIFIC,
    TELEGRAM,
    UNGEGN_1987,
    WIKIPEDIA,
    YANDEX_MAPS,
    YANDEX_MONEY,
)

__version__ = "0.9.0"
__all__ = []



import pytest
import iuliia


@pytest.mark.parametrize(
    "source,expected",
    [
        ("Юлия Щеглова", "Iuliia Shcheglova"),
        (
            "Юлия, съешь ещё этих мягких французских булок из Йошкар-Олы, да выпей алтайского чаю",
            "Iuliia, sesh eshche etikh miagkikh frantcuzskikh bulok iz Ioshkar-Oly, "
            "da vypei altaiskogo chaiu",
        ),
    ],
)
def test_gost_52535(source, expected):
    assert iuliia.translate(source, iuliia.GOST_52535) == expected



"""
GOST 7.79-2000 (aka ISO 9:1995) transliteration schema.
https://dangry.ru/iuliia/gost-779/
"""

from iuliia.mapping import BASE_MAPPING
from iuliia.schema import Schema

MAPPING = {
    **BASE_MAPPING,
    **{
        "ё": "ё",
        "ж": "ž",
        "й": "j",
        "х": "h",
        "ц": "c",
        "ч": "č",
        "ш": "š",
        "щ": "ŝ",
        "ъ": "ʺ",
        "ы": "y",
        "ь": "ʹ",
        "э": "è",
        "ю": "û",
        "я": "â",
    },
}

ALT_MAPPING = {
    **BASE_MAPPING,
    **{
        "ё": "yo",
        "ж": "zh",
        "й": "j",
        "х": "x",
        "ц": "cz",
        "ч": "ch",
        "ш": "sh",
        "щ": "shh",
        "ъ": "``",
        "ы": "y`",
        "ь": "`",
        "э": "е`",
        "ю": "yu",
        "я": "ya",
    },
}

ALT_NEXT_MAPPING = {"це": "c", "ци": "c", "цй": "c", "цы": "c"}

GOST_779 = Schema(MAPPING)
GOST_779_ALT = Schema(ALT_MAPPING, next_mapping=ALT_NEXT_MAPPING)



"""
Iuliia command-line interface
"""

import sys
import iuliia


def main():
    """Transliterate string from command line"""
    if len(sys.argv) < 3:
        print("usage: iuliia SCHEMA SOURCE")
        print("Supported schemas:")
        print("\n".join(iuliia.Schemas.names()))
        sys.exit(1)

    schema_name = sys.argv[1]
    schema = iuliia.Schemas.get(schema_name)
    if schema is None:
        print(f"Schema '{schema_name}' does not exist. Supported schemas:")
        print("\n".join(iuliia.Schemas.names()))
        sys.exit(1)

    source = sys.argv[2]
    result = iuliia.translate(source, schema)
    print(result)


if __name__ == "__main__":
    main()



import pytest
import iuliia


@pytest.mark.parametrize(
    "source,expected",
    [
        (
            "Юлия, съешь ещё этих мягких французских булок из Йошкар-Олы, да выпей алтайского чаю",
            'Julija, s"ešʹ eščë ėtih mjagkih francuzskih bulok iz Joškar-Oly, '
            "da vypej altajskogo čaju",
        ),
    ],
)
def test_iso_9_1954(source, expected):
    assert iuliia.translate(source, iuliia.ISO_9_1954) == expected



"""
Scientific transliteration schema.
https://dangry.ru/iuliia/scientific/
"""

from iuliia.mapping import BASE_MAPPING
from iuliia.schema import Schema

MAPPING = {
    **BASE_MAPPING,
    **{
        "ё": "ё",
        "ж": "ž",
        "й": "j",
        "х": "x",
        "ц": "c",
        "ч": "č",
        "ш": "š",
        "щ": "šč",
        "ъ": "ʺ",
        "ы": "y",
        "ь": "ʹ",
        "э": "è",
        "ю": "ju",
        "я": "ja",
    },
}

SCIENTIFIC = Schema(MAPPING)



import pytest
import iuliia


@pytest.mark.parametrize(
    "source,expected",
    [
        (
            "Юлия, съешь ещё этих мягких французских булок из Йошкар-Олы, да выпей алтайского чаю",
            "Julija, sʺešʹ eščё ètih mjagkih francuzskih bulok iz Joškar-Oly, "
            "da vypej altajskogo čaju",
        ),
        (
            "Россия, город Йошкар-Ола, улица Яна Крастыня",
            "Rossija, gorod Joškar-Ola, ulica Jana Krastynja",
        ),
    ],
)
def test_ungegn_1987(source, expected):
    assert iuliia.translate(source, iuliia.UNGEGN_1987) == expected



import pytest
import iuliia


@pytest.mark.parametrize(
    "source,expected",
    [
        (
            "Юлия, съешь ещё этих мягких французских булок из Йошкар-Олы, да выпей алтайского чаю",
            "Yuliya, sʺeshʹ eshchё étikh myagkikh frantsuzskikh bulok iz Ĭoshkar-Olȳ, "
            "da vȳpeĭ altaĭskogo chayu",
        ),
    ],
)
def test_bs_2979(source, expected):
    assert iuliia.translate(source, iuliia.BS_2979) == expected


@pytest.mark.parametrize(
    "source,expected",
    [
        (
            "Юлия, съешь ещё этих мягких французских булок из Йошкар-Олы, да выпей алтайского чаю",
            "Yuliya, s\"esh' eshche etikh myagkikh frantsuzskikh bulok iz Ioshkar-Oly, "
            "da vypei altaiskogo chayu",
        ),
    ],
)
def test_bs_2979_alt(source, expected):
    assert iuliia.translate(source, iuliia.BS_2979_ALT) == expected



import pytest
import iuliia


@pytest.mark.parametrize(
    "source,expected",
    [
        (
            "Юлия, съешь ещё этих мягких французских булок из Йошкар-Олы, да выпей алтайского чаю",
            "Yuliya, s”yesh’ yeshchё etikh myagkikh frantsuzskikh bulok iz Yoshkar-Oly, "
            "da vypey altayskogo chayu",
        ),
        (
            "Россия, город Йошкар-Ола, улица Яна Крастыня",
            "Rossiya, gorod Yoshkar-Ola, ulitsa Yana Krastynya",
        ),
        ("Елизово", "Yelizovo"),
        ("Чапаевск", "Chapayevsk"),
        ("Мейеровка", "Meyyerovka"),
        ("Юрьев объезд", "Yur’yev ob”yezd"),
        ("Белкино", "Belkino"),
        ("Ёдва", "Yёdva"),
        ("Змииёвка", "Zmiiyёvka"),
        ("Айёган", "Ayyёgan"),
        ("Воробьёво", "Vorob’yёvo"),
        ("Кебанъёль", "Keban”yёl’"),
        ("Озёрный", "Ozёrnyy"),
        ("Тыайа", "Ty·ay·a"),
        ("Сайылык", "Say·ylyk"),
        ("Ойусардах", "Oy·usardakh"),
        ("Йошкар-Ола", "Yoshkar-Ola"),
        ("Бийск", "Biysk"),
        ("Тыэкан", "Ty·ekan"),
        ("Суык-Су", "Su·yk-Su"),
        ("Тында", "Tynda"),
        ("Улан-Удэ", "Ulan-Ud·e"),
        ("Электрогорск", "Elektrogorsk"),
        ("Руэм", "Ruem"),
        ("Вяртсиля", "Vyart·silya"),
        ("Ташчишма", "Tash·chishma"),
    ],
)
def test_bgn_pcgn(source, expected):
    assert iuliia.translate(source, iuliia.BGN_PCGN) == expected


@pytest.mark.parametrize(
    "source,expected",
    [
        (
            "Юлия, съешь ещё этих мягких французских булок из Йошкар-Олы, да выпей алтайского чаю",
            "Yuliya, s”yesh’ yeshchё etikh myagkikh frantsuzskikh bulok iz Yoshkar-Oly, "
            "da vypey altayskogo chayu",
        ),
        (
            "Россия, город Йошкар-Ола, улица Яна Крастыня",
            "Rossiya, gorod Yoshkar-Ola, ulitsa Yana Krastynya",
        ),
        ("Елизово", "Yelizovo"),
        ("Чапаевск", "Chapayevsk"),
        ("Мейеровка", "Meyyerovka"),
        ("Юрьев объезд", "Yur’yev ob”yezd"),
        ("Белкино", "Belkino"),
        ("Ёдва", "Yёdva"),
        ("Змииёвка", "Zmiiyёvka"),
        ("Айёган", "Ayyёgan"),
        ("Воробьёво", "Vorob’yёvo"),
        ("Кебанъёль", "Keban”yёl’"),
        ("Озёрный", "Ozёrnyy"),
        ("Тыайа", "Tyaya"),
        ("Сайылык", "Sayylyk"),
        ("Ойусардах", "Oyusardakh"),
        ("Йошкар-Ола", "Yoshkar-Ola"),
        ("Бийск", "Biysk"),
        ("Тыэкан", "Tyekan"),
        ("Суык-Су", "Suyk-Su"),
        ("Тында", "Tynda"),
        ("Улан-Удэ", "Ulan-Ude"),
        ("Электрогорск", "Elektrogorsk"),
        ("Руэм", "Ruem"),
        ("Вяртсиля", "Vyartsilya"),
        ("Ташчишма", "Tashchishma"),
    ],
)
def test_bgn_pcgn_alt(source, expected):
    assert iuliia.translate(source, iuliia.BGN_PCGN_ALT) == expected



"""
British Standard 2979:1958 transliteration schema.
https://dangry.ru/iuliia/bs-2979/
"""

from iuliia.mapping import BASE_MAPPING
from iuliia.schema import Schema

MAPPING = {
    **BASE_MAPPING,
    **{
        "ё": "ё",
        "ж": "zh",
        "й": "ĭ",
        "х": "kh",
        "ц": "ts",
        "ч": "ch",
        "ш": "sh",
        "щ": "shch",
        "ъ": "ʺ",
        # This schema defines two alternative translations for `Ы`:
        #   - `Ы` → `Ȳ` (used by the Oxford University Press)
        #   - `Ы` → `UI` (used by the British Library).
        # `iuliia` uses `Ы` → `Ȳ`.
        "ы": "ȳ",
        "ь": "ʹ",
        "э": "é",
        "ю": "yu",
        "я": "ya",
    },
}

ALT_MAPPING = {
    **BASE_MAPPING,
    **{
        "ё": "e",
        "ж": "zh",
        "й": "i",
        "х": "kh",
        "ц": "ts",
        "ч": "ch",
        "ш": "sh",
        "щ": "shch",
        "ъ": '"',
        "ы": "y",
        "ь": "'",
        "э": "e",
        "ю": "yu",
        "я": "ya",
    },
}

ENDING_MAPPING = {"ий": "y", "ый": "y"}

BS_2979 = Schema(MAPPING, ending_mapping=ENDING_MAPPING)
BS_2979_ALT = Schema(ALT_MAPPING, ending_mapping=ENDING_MAPPING)



"""
Letter mapping for transliteration schema.
"""

BASE_MAPPING = {
    "а": "a",
    "б": "b",
    "в": "v",
    "г": "g",
    "д": "d",
    "е": "e",
    "з": "z",
    "и": "i",
    "к": "k",
    "л": "l",
    "м": "m",
    "н": "n",
    "о": "o",
    "п": "p",
    "р": "r",
    "с": "s",
    "т": "t",
    "у": "u",
    "ф": "f",
}


class Mapping:
    """Letter map for transliteration schema."""

    def __init__(self, mapping: dict):
        self.map = mapping.copy()

    def get(self, key, default=None):
        """Return mapped value for ``key`` if key is in the map, else ``default``."""
        return self.map.get(key, default)

    def __str__(self):
        return str(self.map)

    def __repr__(self):
        return repr(self.map)


class LetterMapping(Mapping):
    """Mapping for individual letters."""

    def __init__(self, mapping: dict):
        super().__init__(mapping)
        upper_map = {key.capitalize(): value.capitalize() for key, value in mapping.items()}
        self.map.update(upper_map)


class PrevMapping(Mapping):
    """Mapping for letters with respect to previous sibling."""

    def __init__(self, mapping: dict):
        super().__init__(mapping)
        upper_map_1 = {key.capitalize(): value for key, value in mapping.items()}
        upper_map_2 = {key.upper(): value.capitalize() for key, value in mapping.items()}
        self.map.update(upper_map_1)
        self.map.update(upper_map_2)


class NextMapping(Mapping):
    """Mapping for letters with respect to next sibling."""

    def __init__(self, mapping: dict):
        super().__init__(mapping)
        upper_map_1 = {key.capitalize(): value.capitalize() for key, value in mapping.items()}
        upper_map_2 = {key.upper(): value.capitalize() for key, value in mapping.items()}
        self.map.update(upper_map_1)
        self.map.update(upper_map_2)


class EndingMapping(Mapping):
    """Mapping for word endings."""

    def __init__(self, mapping: dict):
        super().__init__(mapping)
        upper_map = {key.upper(): value.upper() for key, value in mapping.items()}
        self.map.update(upper_map)



"""
Wikipedia transliteration schema.
https://dangry.ru/iuliia/wikipedia/
"""

from iuliia.mapping import BASE_MAPPING
from iuliia.schema import Schema

MAPPING = {
    **BASE_MAPPING,
    **{
        "ё": "yo",
        "ж": "zh",
        "й": "y",
        "х": "kh",
        "ц": "ts",
        "ч": "ch",
        "ш": "sh",
        "щ": "shch",
        "ъ": "",
        "ы": "y",
        "ь": "",
        "э": "e",
        "ю": "yu",
        "я": "ya",
    },
}

PREV_MAPPING = {
    "е": "ye",
    "ае": "ye",
    "ие": "ye",
    "ое": "ye",
    "уе": "ye",
    "эе": "ye",
    "юе": "ye",
    "яе": "ye",
    "ье": "ye",
    "ъе": "ye",
}

NEXT_MAPPING = {
    "ъа": "y",
    "ъи": "y",
    "ъо": "y",
    "ъу": "y",
    "ъы": "y",
    "ъэ": "y",
    "ьа": "y",
    "ьи": "y",
    "ьо": "y",
    "ьу": "y",
    "ьы": "y",
    "ьэ": "y",
}

ENDING_MAPPING = {
    "ий": "y",
    "ый": "y",
}

WIKIPEDIA = Schema(
    MAPPING, prev_mapping=PREV_MAPPING, next_mapping=NEXT_MAPPING, ending_mapping=ENDING_MAPPING,
)



import pytest
import iuliia


@pytest.mark.parametrize(
    "source,expected",
    [
        (
            "Юлия, съешь ещё этих мягких французских булок из Йошкар-Олы, да выпей алтайского чаю",
            "Julija, sʺešʹ eščë ėtih mjagkih francuzskih bulok iz Joškar-Oly, "
            "da vypej altajskogo čaju",
        ),
    ],
)
def test_iso_9_1968(source, expected):
    assert iuliia.translate(source, iuliia.ISO_9_1968) == expected


@pytest.mark.parametrize(
    "source,expected",
    [
        (
            "Юлия, съешь ещё этих мягких французских булок из Йошкар-Олы, да выпей алтайского чаю",
            "Yulyya, sʺeshʹ eshchë ėtykh myagkykh frantsuzskykh bulok yz Ĭoshkar-Oly, "
            "da vypeĭ altaĭskogo chayu",
        ),
    ],
)
def test_iso_9_1968_alt(source, expected):
    assert iuliia.translate(source, iuliia.ISO_9_1968_ALT) == expected



import pytest
import iuliia


@pytest.mark.parametrize(
    "source,expected",
    [
        ("Юлия Щеглова", "Iuliia Shcheglova"),
        ("Гайа Васильева", "Gaia Vasileva"),
        ("Андрей Видный", "Andrei Vidnyi"),
        ("Артём Краевой", "Artem Kraevoi"),
        ("Мадыр Чёткий", "Madyr Chetkii"),
        ("Оксана Клеёнкина", "Oksana Kleenkina"),
        ("Игорь Ильин", "Igor Ilin"),
        ("Ян Разъездной", "Ian Razieezdnoi"),
    ],
)
def test_icao_doc_9303(source, expected):
    assert iuliia.translate(source, iuliia.ICAO_DOC_9303) == expected



import pytest
import iuliia


@pytest.mark.parametrize(
    "source,expected",
    [
        (
            "Юлия, съешь ещё этих мягких французских булок из Йошкар-Олы, да выпей алтайского чаю",
            "Julija, sʺešʹ eščё ètix mjagkix francuzskix bulok iz Joškar-Oly, "
            "da vypej altajskogo čaju",
        ),
    ],
)
def test_scientific(source, expected):
    assert iuliia.translate(source, iuliia.SCIENTIFIC) == expected



"""
Telegram transliteration schema.
https://dangry.ru/iuliia/telegram/
"""

from iuliia.mapping import BASE_MAPPING
from iuliia.schema import Schema

MAPPING = {
    **BASE_MAPPING,
    **{
        "ё": "e",
        "ж": "j",
        "й": "i",
        "х": "h",
        "ц": "c",
        "ч": "ch",
        "ш": "sh",
        "щ": "sc",
        "ъ": "",
        "ы": "y",
        "ь": "",
        "э": "e",
        "ю": "iu",
        "я": "ia",
    },
}

TELEGRAM = Schema(MAPPING)



"""
GOST 16876-71 (aka GOST 1983) transliteration schema.
https://dangry.ru/iuliia/gost-16876/
"""

from iuliia.mapping import BASE_MAPPING
from iuliia.schema import Schema

MAPPING = {
    **BASE_MAPPING,
    **{
        "ё": "ё",
        "ж": "ž",
        "й": "j",
        "х": "h",
        "ц": "c",
        "ч": "č",
        "ш": "š",
        "щ": "ŝ",
        "ъ": "ʺ",
        "ы": "y",
        "ь": "ʹ",
        "э": "è",
        "ю": "û",
        "я": "â",
    },
}

ALT_MAPPING = {
    **BASE_MAPPING,
    **{
        "ё": "jo",
        "ж": "zh",
        "й": "jj",
        "х": "kh",
        "ц": "c",
        "ч": "ch",
        "ш": "sh",
        "щ": "shh",
        "ъ": '"',
        "ы": "y",
        "ь": "'",
        "э": "eh",
        "ю": "ju",
        "я": "ja",
    },
}

GOST_16876 = Schema(MAPPING)
GOST_16876_ALT = Schema(ALT_MAPPING)



"""
GOST R 52290-2004 transliteration schema.
https://dangry.ru/iuliia/gost-52290/
"""

from iuliia.mapping import BASE_MAPPING
from iuliia.schema import Schema

MAPPING = {
    **BASE_MAPPING,
    **{
        "ё": "yo",
        "ж": "zh",
        "й": "y",
        "х": "kh",
        "ц": "ts",
        "ч": "ch",
        "ш": "sh",
        "щ": "shch",
        "ъ": "'",
        "ы": "y",
        "ь": "'",
        "э": "e",
        "ю": "yu",
        "я": "ya",
    },
}

PREV_MAPPING = {
    # е
    "ае": "ye",
    "ее": "ye",
    "ёе": "ye",
    "ие": "ye",
    "ое": "ye",
    "уе": "ye",
    "ъе": "ye",
    "ые": "ye",
    "ье": "ye",
    "эе": "ye",
    "юе": "ye",
    "яе": "ye",
    # ё
    "бё": "ye",
    "вё": "ye",
    "гё": "ye",
    "дё": "ye",
    "зё": "ye",
    "кё": "ye",
    "лё": "ye",
    "мё": "ye",
    "нё": "ye",
    "пё": "ye",
    "рё": "ye",
    "сё": "ye",
    "тё": "ye",
    "фё": "ye",
    "хё": "ye",
    "цё": "ye",
    "жё": "e",
    "чё": "e",
    "шё": "e",
    "щё": "e",
}

NEXT_MAPPING = {
    "ъе": "",
    "ье": "",
    "ъё": "",
    "ьё": "",
}

GOST_52290 = Schema(MAPPING, prev_mapping=PREV_MAPPING, next_mapping=NEXT_MAPPING)



"""
Yandex.Maps transliteration schema.
https://dangry.ru/iuliia/yandex-maps/
"""

from iuliia.mapping import BASE_MAPPING
from iuliia.schema import Schema

MAPPING = {
    **BASE_MAPPING,
    **{
        "ё": "yo",
        "ж": "zh",
        "й": "y",
        "х": "kh",
        "ц": "ts",
        "ч": "ch",
        "ш": "sh",
        "щ": "sch",
        "ъ": "",
        "ы": "y",
        "ь": "",
        "э": "e",
        "ю": "yu",
        "я": "ya",
    },
}

NEXT_MAPPING = {"ъе": "y"}

ENDING_MAPPING = {"ый": "iy"}

YANDEX_MAPS = Schema(MAPPING, next_mapping=NEXT_MAPPING, ending_mapping=ENDING_MAPPING)



"""
Specific transliteration schemas.
"""

from enum import Enum
from .alalc import ALA_LC, ALA_LC_ALT
from .bgnpcgn import BGN_PCGN, BGN_PCGN_ALT
from .bs2979 import BS_2979, BS_2979_ALT
from .gost16876 import GOST_16876, GOST_16876_ALT
from .gost52290 import GOST_52290
from .gost52535 import GOST_52535
from .gost7034 import GOST_7034
from .gost779 import GOST_779, GOST_779_ALT
from .icaodoc9303 import ICAO_DOC_9303
from .iso91954 import ISO_9_1954
from .iso91968 import ISO_9_1968, ISO_9_1968_ALT
from .mvd310 import MVD_310, MVD_310_FR
from .mvd782 import MVD_782
from .scientific import SCIENTIFIC
from .telegram import TELEGRAM
from .ungegn1987 import UNGEGN_1987
from .wikipedia import WIKIPEDIA
from .yandexmaps import YANDEX_MAPS
from .yandexmoney import YANDEX_MONEY


class Schemas(Enum):
    """All supported transliteration schemas."""

    ala_lc = ALA_LC
    ala_lc_alt = ALA_LC_ALT
    bgn_pcgn = BGN_PCGN
    bgn_pcgn_alt = BGN_PCGN_ALT
    bs_2979 = BS_2979
    bs_2979_alt = BS_2979_ALT
    gost_16876 = GOST_16876
    gost_16876_alt = GOST_16876_ALT
    gost_52290 = GOST_52290
    gost_52535 = GOST_52535
    gost_7034 = GOST_7034
    gost_779 = GOST_779
    gost_779_alt = GOST_779_ALT
    icao_doc_9303 = ICAO_DOC_9303
    iso_9 = GOST_779
    iso_9_1954 = ISO_9_1954
    iso_9_1968 = ISO_9_1968
    iso_9_1968_alt = ISO_9_1968_ALT
    mvd_310 = MVD_310
    mvd_310_fr = MVD_310_FR
    mvd_782 = MVD_782
    scientific = SCIENTIFIC
    telegram = TELEGRAM
    ungegn_1987 = UNGEGN_1987
    wikipedia = WIKIPEDIA
    yandex_maps = YANDEX_MAPS
    yandex_money = YANDEX_MONEY

    @classmethod
    def names(cls):
        """Return sorted list of all supported schemas."""
        return sorted(item.name for item in cls)

    @classmethod
    def get(cls, name):
        """Return schema by its name or ``None`` if nothing found."""
        item = cls.__members__.get(name)
        return item.value if item else None



import pytest
from iuliia.mapping import Mapping, LetterMapping, PrevMapping, NextMapping, EndingMapping


@pytest.fixture
def mapping():
    return Mapping({"a": "x", "b": "yy"})


def test_map(mapping):
    assert mapping.map == {"a": "x", "b": "yy"}


def test_get(mapping):
    assert mapping.get("a") == "x"
    assert mapping.get("b") == "yy"
    assert mapping.get("c") is None
    assert mapping.get("d", "d") == "d"


def test_str(mapping):
    assert str(mapping) == str(mapping.map)
    assert repr(mapping) == repr(mapping.map)


def test_letter_mapping():
    mapping = LetterMapping({"a": "x", "b": "yy"})
    assert mapping.map == {"a": "x", "A": "X", "b": "yy", "B": "Yy"}


def test_prev_mapping():
    mapping = PrevMapping({"ax": "xx", "bx": "xxx"})
    assert mapping.map == {
        "ax": "xx",
        "Ax": "xx",
        "AX": "Xx",
        "bx": "xxx",
        "Bx": "xxx",
        "BX": "Xxx",
    }


def test_next_mapping():
    mapping = NextMapping({"xa": "xx", "xb": "xxx"})
    assert mapping.map == {
        "xa": "xx",
        "Xa": "Xx",
        "XA": "Xx",
        "xb": "xxx",
        "Xb": "Xxx",
        "XB": "Xxx",
    }


def test_ending_mapping():
    mapping = EndingMapping({"aa": "xx", "bb": "yy"})
    assert mapping.map == {"aa": "xx", "AA": "XX", "bb": "yy", "BB": "YY"}



#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['iuliia', 'iuliia.schemas']

package_data = \
{'': ['*']}

entry_points = \
{'console_scripts': ['iuliia = iuliia.__main__:main']}

setup(name='iuliia',
      version='0.9.0',
      description='Transliterate Cyrillic → Latin in every possible way',
      author='Anton Zhiyanov',
      author_email='m@antonz.ru',
      url='https://github.com/nalgeon/iuliia-py',
      packages=packages,
      package_data=package_data,
      entry_points=entry_points,
      python_requires='>=3.6',
     )



import pytest
import iuliia


@pytest.mark.parametrize(
    "source,expected",
    [
        (
            "Юлия, съешь ещё этих мягких французских булок из Йошкар-Олы, да выпей алтайского чаю",
            "Yuliya, syesh yeshchyo etikh myagkikh frantsuzskikh bulok iz Yoshkar-Oly, "
            "da vypey altayskogo chayu",
        ),
        (
            "Россия, город Йошкар-Ола, улица Яна Крастыня",
            "Rossiya, gorod Yoshkar-Ola, ulitsa Yana Krastynya",
        ),
        ("Ельцин", "Yeltsin"),
        ("Раздольное", "Razdolnoye"),
        ("Юрьев", "Yuryev"),
        ("Белкин", "Belkin"),
        ("Бийск", "Biysk"),
        ("Подъярский", "Podyarsky"),
        ("Мусийкъонгийкоте", "Musiykyongiykote"),
        ("Давыдов", "Davydov"),
        ("Усолье", "Usolye"),
        ("Выхухоль", "Vykhukhol"),
        ("Дальнегорск", "Dalnegorsk"),
        ("Ильинский", "Ilyinsky"),
        ("Красный", "Krasny"),
        ("Великий", "Veliky"),
        ("Набережные Челны", "Naberezhnye Chelny"),
    ],
)
def test_wikipedia(source, expected):
    assert iuliia.translate(source, iuliia.WIKIPEDIA) == expected



"""
GOST R 7.0.34-2014 transliteration schema.
http://localhost:3000/iuliia/gost-7034/
"""

from iuliia.mapping import BASE_MAPPING
from iuliia.schema import Schema

# This schema defines alternatives for many letters, but does not specify when to use which.
# Therefore, `iuliia` uses the first of suggested translations for each such letter.
MAPPING = {
    **BASE_MAPPING,
    **{
        "е": "e",  # (ye)
        "ё": "yo",  # (jo)
        "ж": "zh",
        "й": "j",  # (i,y)
        "х": "x",  # (kh)
        "ц": "c",  # (tz,cz)
        "ч": "ch",
        "ш": "sh",
        "щ": "shh",
        "ъ": "''",  # (empty)
        "ы": "y",
        "ь": "'",  # (empty)
        "э": "e",
        "ю": "yu",  # (ju)
        "я": "ya",  # (ja)
    },
}

GOST_7034 = Schema(MAPPING)



import pytest
import iuliia


@pytest.mark.parametrize(
    "source,expected",
    [
        ("Юлия Щеглова", "Yuliya Scheglova"),
        ("Иван Брызгальский", "Ivan Bryzgalskii"),
        ("Ксения Стрый", "Kseniya Stryi"),
    ],
)
def test_yandex_money(source, expected):
    assert iuliia.translate(source, iuliia.YANDEX_MONEY) == expected



"""
GOST R 52535.1-2006 transliteration schema.
http://localhost:3000/iuliia/gost-52535/
"""

from iuliia.mapping import BASE_MAPPING
from iuliia.schema import Schema

MAPPING = {
    **BASE_MAPPING,
    **{
        "ё": "e",
        "ж": "zh",
        "й": "i",
        "х": "kh",
        "ц": "tc",
        "ч": "ch",
        "ш": "sh",
        "щ": "shch",
        "ъ": "",
        "ы": "y",
        "ь": "",
        "э": "e",
        "ю": "iu",
        "я": "ia",
    },
}

GOST_52535 = Schema(MAPPING)
